@@ -20,7 +20,8 @@
   const { parseFormData, formDataFromEntries, encodeFormData } =
     globalThis.__bootstrap.formData;
   const mimesniff = globalThis.__bootstrap.mimesniff;
-  const { isReadableStreamDisturbed } = globalThis.__bootstrap.streams;
+  const { isReadableStreamDisturbed, errorReadableStream } =
+    globalThis.__bootstrap.streams;
 
   class InnerBody {
     /** @type {ReadableStream<Uint8Array> | { body: Uint8Array, consumed: boolean }} */
@@ -106,6 +107,22 @@
       }
     }
 
+    cancel(error) {
+      if (this.streamOrStatic instanceof ReadableStream) {
+        this.streamOrStatic.cancel(error);
+      } else {
+        this.streamOrStatic.consumed = true;
+      }
+    }
+
+    error(error) {
+      if (this.streamOrStatic instanceof ReadableStream) {
+        errorReadableStream(this.streamOrStatic, error);
+      } else {
+        this.streamOrStatic.consumed = true;
+      }
+    }
+
     /**
      * @returns {InnerBody}
      */
@@ -223,8 +240,6 @@
     return Object.defineProperties(prototype.prototype, mixin);
   }
 
-  const decoder = new TextDecoder();
-
   /**
    * https://fetch.spec.whatwg.org/#concept-body-package-data
    * @param {Uint8Array} bytes
@@ -263,14 +278,12 @@
         throw new TypeError("Missing content type");
       }
       case "JSON":
-        return JSON.parse(decoder.decode(bytes));
+        return JSON.parse(core.decode(bytes));
       case "text":
-        return decoder.decode(bytes);
+        return core.decode(bytes);
     }
   }
 
-  const encoder = new TextEncoder();
-
   /**
    * @param {BodyInit} object
    * @returns {{body: InnerBody, contentType: string | null}}
@@ -305,10 +318,10 @@
       length = res.body.byteLength;
       contentType = res.contentType;
     } else if (object instanceof URLSearchParams) {
-      source = encoder.encode(object.toString());
+      source = core.encode(object.toString());
       contentType = "application/x-www-form-urlencoded;charset=UTF-8";
     } else if (typeof object === "string") {
-      source = encoder.encode(object);
+      source = core.encode(object);
       contentType = "text/plain;charset=UTF-8";
     } else if (object instanceof ReadableStream) {
       stream = object;
